import 'package:flutter/material.dart';
import 'dart:math' as math;

class CustomCircularIndicator extends StatefulWidget {
  const CustomCircularIndicator({
    Key? key,
    required this.strokeWidth,
    required this.startAngle,
    required this.sweepAngle,
    required this.radius,
    required this.firstCircleColor,
    required this.secondCircleColor,
    required this.thirdCircleColor,
    required this.width,
    required this.height,
  }) : super(key: key);

  final double strokeWidth;
  final double startAngle;
  final double sweepAngle;
  final Color firstCircleColor;
  final Color secondCircleColor;
  final Color thirdCircleColor;
  final double radius;
  final double width;
  final double height;

  @override
  _CustomCircularIndicatorState createState() =>
      _CustomCircularIndicatorState();
}

class _CustomCircularIndicatorState extends State<CustomCircularIndicator>
    with TickerProviderStateMixin {
  late final AnimationController _blueIndicatorController;
  late final Animation _blueIndicatorAnimation;

  late final AnimationController _redIndicatorController;
  late final Animation _redIndicatorAnimation;

  late final AnimationController _yellowIndicatorController;
  late final Animation _yellowIndicatorAnimation;

  @override
  void initState() {
    _blueIndicatorController = AnimationController(
      duration: const Duration(milliseconds: 1500),
      vsync: this,
    )..repeat();

    _redIndicatorController = AnimationController(
      duration: const Duration(milliseconds: 1200),
      vsync: this,
    )..repeat();

    _yellowIndicatorController = AnimationController(
      duration: const Duration(milliseconds: 1000),
      vsync: this,
    )..repeat();

    _blueIndicatorAnimation =
        Tween(begin: 0, end: 2 * math.pi).animate(_blueIndicatorController);
    super.initState();

    _redIndicatorAnimation =
        Tween(begin: 0, end: 2 * math.pi).animate(_redIndicatorController);
    super.initState();

    _yellowIndicatorAnimation =
        Tween(begin: 0, end: 2 * math.pi).animate(_yellowIndicatorController);
    super.initState();
  }

  @override
  void dispose() {
    _blueIndicatorController.dispose();
    _redIndicatorController.dispose();
    _yellowIndicatorController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Stack(children: [
      AnimatedBuilder(
        animation: _blueIndicatorAnimation,
        child: CustomPaint(
          painter: ProgressIndicatorPainter(
            strokeWidth: widget.strokeWidth,
            startAngle: widget.startAngle,
            sweepAngle: widget.sweepAngle,
            color: widget.firstCircleColor,
            radius: widget.radius / 2,
          ),
          child: SizedBox(
            width: widget.width,
            height: widget.height,
          ),
        ),
        builder: (BuildContext context, Widget? child) {
          return Transform.rotate(
            angle: _blueIndicatorAnimation.value,
            child: child,
          );
        },
      ),
      AnimatedBuilder(
        animation: _redIndicatorAnimation,
        child: CustomPaint(
          painter: ProgressIndicatorPainter(
            strokeWidth: widget.strokeWidth,
            startAngle: widget.startAngle,
            sweepAngle: widget.sweepAngle,
            color: widget.secondCircleColor,
            radius: widget.radius / 2.3,
          ),
          child: SizedBox(
            width: widget.width,
            height: widget.height,
          ),
        ),
        builder: (BuildContext context, Widget? child) {
          return Transform.rotate(
            angle: _redIndicatorAnimation.value,
            child: child,
          );
        },
      ),
      AnimatedBuilder(
        animation: _yellowIndicatorAnimation,
        child: CustomPaint(
          painter: ProgressIndicatorPainter(
            strokeWidth: widget.strokeWidth,
            startAngle: widget.startAngle,
            sweepAngle: widget.sweepAngle,
            color: widget.thirdCircleColor,
            radius: widget.radius / 2.8,
          ),
          child: SizedBox(
            width: widget.width,
            height: widget.height,
          ),
        ),
        builder: (BuildContext context, Widget? child) {
          return Transform.rotate(
            angle: _yellowIndicatorAnimation.value,
            child: child,
          );
        },
      ),
    ]);
  }
}

class ProgressIndicatorPainter extends CustomPainter {
  const ProgressIndicatorPainter({
    required this.strokeWidth,
    required this.startAngle,
    required this.sweepAngle,
    required this.color,
    required this.radius,
  }) : super();

  final double strokeWidth;
  final double startAngle;
  final double sweepAngle;
  final Color color;
  final double radius;

  @override
  void paint(Canvas canvas, Size size) {
    final bluePaint = Paint()
      ..color = color
      ..style = PaintingStyle.stroke
      ..strokeCap = StrokeCap.square
      ..strokeWidth = strokeWidth;

    final center = Offset(size.width / 2, size.height / 2);

    canvas.drawArc(Rect.fromCircle(center: center, radius: radius), startAngle,
        sweepAngle, false, bluePaint);
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) => false;
}
